package com.ningdatech.pmapi.ding.controller;

import com.alibaba.fastjson.JSON;
import com.ningdatech.pmapi.common.util.CryptUtils;
import com.ningdatech.pmapi.common.util.RefreshKeyUtil;
import com.ningdatech.pmapi.ding.task.EmployeeBatchGetTask;
import com.ningdatech.pmapi.ding.task.GovBusinessStripsTask;
import com.ningdatech.pmapi.ding.task.OrganizationBatchGetTask;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.security.NoSuchAlgorithmException;

/**
 * <p>
 * PullDingInfoController
 * </p>
 *
 * @author WendyYang
 * @since 2023/6/2
 **/
@RestController
@RequestMapping("/api/v1/zwdd/pull")
@RequiredArgsConstructor
public class DingInfoPullController {

    private final EmployeeBatchGetTask employeeBatchGetTask;
    private final GovBusinessStripsTask businessStripsTask;
    private final OrganizationBatchGetTask organizationBatchGetTask;


    @GetMapping("/employee")
    public void employeeBatchGet() {
        employeeBatchGetTask.batchGetEmployeeTask();
    }

    @GetMapping("/businessStrips")
    public void businessStripsBatchGet() {
        businessStripsTask.batchGetGovBusinessStripsTask();
    }

    @GetMapping("/organization")
    public void organizationBatchGet() {
        organizationBatchGetTask.batchGetOrganizationTask();
    }

    @GetMapping("/employeeByRegionCode/{regionCode}")
    public void employeeBatchGetByRegionCode(@PathVariable String regionCode) {
        employeeBatchGetTask.batchGetEmployeeTaskByRegionCode(regionCode);
    }

    @GetMapping("/employeeByOrdCode/{orgCode}")
    public void employeeBatchGetByOrdCode(@PathVariable String orgCode) {
        employeeBatchGetTask.batchGetEmployeeTaskByOrdCode(orgCode);
    }

    @GetMapping("/getEmployeeByCode/{empCode}")
    public void getEmployeeByCode(@PathVariable String empCode) {
        employeeBatchGetTask.getEmployeeByCode(empCode);
    }

    @GetMapping("/getBatchEmployeeByCode/{orgCode}")
    public void getBatchEmployeeByCode(@PathVariable String orgCode) {
        employeeBatchGetTask.getBatchEmployeeByCode(orgCode);
    }

    @GetMapping("/test-app")
    public void testApp() throws NoSuchAlgorithmException {
        long timestamp = System.currentTimeMillis();
        String areaCode = "331121";
        String appKey = "A331101453557202109017383";
        String appScret = "496f0f2a19994f76b4fd9dae087366c7";

        String requestSecret = RefreshKeyUtil.getRequestSecret(appKey, appScret);
        String capCode = CryptUtils.MD5Encode(timestamp + areaCode);
        String capTime = String.valueOf(timestamp);
        String sign = CryptUtils.MD5Encode(appKey + requestSecret + timestamp);
        String url = "https://interface.zjzwfw.gov.cn/gateway/api/proxy/001003001029/dataSharing/94wbaL1I1Pbz0648.htm?requestTime=" + timestamp +
                "&sign=" + sign + "&appKey=" + appKey + "&capCode=" + capCode + "&capTime=" + capTime +
                "&baseProjSys=测试应用&areaCode=331121&baseProjName=nsl-丽水演示项目&baseProjId=331100230130112233001" +
                "&isEffective=1&baseProjSysCode=25083657";
        RestTemplate restTemplate = new RestTemplate();
        ResponseEntity<String> forEntity = restTemplate.getForEntity(url, String.class);

        System.out.println(JSON.toJSONString(forEntity));
    }

    @GetMapping("/getToken")
    public String getToken() {
       return employeeBatchGetTask.getToken();
    }
}
