package com.ningdatech.pmapi.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.service.FileService;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.common.constant.CommonConst;
import com.ningdatech.pmapi.common.constant.RegionConst;
import com.ningdatech.pmapi.common.enumeration.CommonEnum;
import com.ningdatech.pmapi.common.enumeration.ProjectProcessStageEnum;
import com.ningdatech.pmapi.common.helper.RegionCacheHelper;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.model.entity.ExcelExportWriter;
import com.ningdatech.pmapi.common.statemachine.util.StateMachineUtils;
import com.ningdatech.pmapi.common.util.BizUtils;
import com.ningdatech.pmapi.common.util.ExcelDownUtil;
import com.ningdatech.pmapi.datascope.model.DataScopeDTO;
import com.ningdatech.pmapi.datascope.utils.DataScopeUtil;
import com.ningdatech.pmapi.filemanage.model.entity.ProjectApplyBorrow;
import com.ningdatech.pmapi.filemanage.service.INdProjectApplyBorrowService;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.projectdeclared.model.dto.DelayedApplyDTO;
import com.ningdatech.pmapi.projectdeclared.model.entity.Contract;
import com.ningdatech.pmapi.projectdeclared.model.entity.PaymentPlan;
import com.ningdatech.pmapi.projectdeclared.model.entity.PreInsAcceptancePerson;
import com.ningdatech.pmapi.projectdeclared.model.entity.Purchase;
import com.ningdatech.pmapi.projectdeclared.model.vo.ContractVO;
import com.ningdatech.pmapi.projectdeclared.model.vo.PaymentPlanVO;
import com.ningdatech.pmapi.projectdeclared.model.vo.PreInsAcceptancePersonVO;
import com.ningdatech.pmapi.projectdeclared.model.vo.PurchaseVO;
import com.ningdatech.pmapi.projectdeclared.service.IContractService;
import com.ningdatech.pmapi.projectdeclared.service.IPaymentPlanService;
import com.ningdatech.pmapi.projectdeclared.service.IPreInsAcceptancePersonService;
import com.ningdatech.pmapi.projectdeclared.service.IPurchaseService;
import com.ningdatech.pmapi.projectdeclared.utils.GenerateProjectCodeUtil;
import com.ningdatech.pmapi.projectlib.enumeration.InstTypeEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.handle.ProcessExecuteChainHandle;
import com.ningdatech.pmapi.projectlib.helper.ProjectHelper;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectApplicationDTO;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.*;
import com.ningdatech.pmapi.projectlib.model.req.ProjectApplicationListReq;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.projectlib.model.vo.AnnualAmountVO;
import com.ningdatech.pmapi.projectlib.model.vo.ProjectApplicationVO;
import com.ningdatech.pmapi.projectlib.model.vo.ProjectDetailVO;
import com.ningdatech.pmapi.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.pmapi.projectlib.service.*;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.wflow.exception.BusinessException;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * ProjectLibManage
 * </p>
 *
 * @author WendyYang
 * @since 14:19 2023/2/1
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class ProjectLibManage {

    private final IProjectService projectService;
    private final IProjectApplicationService applicationService;
    private final IProjectRenewalFundDeclarationService renewalFundDeclarationService;
    private final ProcessExecuteChainHandle processExecuteHandle;
    private final RegionCacheHelper regionCacheHelper;
    private final FileService fileService;
    private final IProjectApplicationService projectApplicationService;
    private final GenerateProjectCodeUtil generateProjectCodeUtil;
    private final IProjectInstService projectInstService;
    private final StateMachineUtils stateMachineUtils;
    private final INdProjectDelayApplyService projectDelayApplyService;
    private final INdProjectApplyBorrowService projectApplyBorrowService;
    private final IDingOrganizationService dingOrganizationService;
    private final IContractService contractService;

    private final IPurchaseService purchaseService;

    private final IPreInsAcceptancePersonService acceptancePersonService;

    private final IPaymentPlanService paymentPlanService;

    public PageVo<ProjectLibListItemVO> projectLibList(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(req);
        query.eq(Project::getNewest,Boolean.TRUE);
        Page<Project> page = projectService.page(req.page(), query);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            return item;
        });
        return PageVo.of(records, total);
    }

    public PageVo<ProjectLibListItemVO> projectLibListWithPermission(ProjectListReq req,UserFullInfoDTO user) {
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(req);
        user = buildProjectLibPermission(query,user);
        //项目查最新
        query.eq(Project::getNewest,Boolean.TRUE);
        Page<Project> page = projectService.page(req.page(), query);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        UserFullInfoDTO finalUser = user;
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            if (finalUser.getIsOrgAdmin() &&
                    ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode().equals(item.getStatus())
                    && StringUtils.isNotBlank(w.getSuperOrgCode()) && StringUtils.isNotBlank(finalUser.getEmpPosUnitCode())
                    && w.getSuperOrgCode().equals(finalUser.getEmpPosUnitCode())) {
                item.setCanPreDeclared(Boolean.TRUE);
            }
            return item;
        });
        return PageVo.of(records, total);
    }

    public List<Project> projectList(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(req);
        return projectService.list(query);
    }

    /**
     * 申报新项目时 保存项目信息和其它相关联的信息
     * @param projectDto
     * @param instanceId
     * @param employeeCode
     * @return
     */
    public Project saveProjectInDeclared(ProjectDTO projectDto, String instanceId,
                                String employeeCode) {
        Project project = saveProjectNewVersion(projectDto,instanceId,employeeCode);
        //保存项目和实例的关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setProjectId(project.getId());
        projectInst.setInstCode(instanceId);
        projectInst.setCreatOn(LocalDateTime.now());
        projectInst.setUpdateOn(LocalDateTime.now());
        projectInst.setInstType(ProjectProcessStageEnum.ORG_INTERNAL_APPROVAL_PROCESS.getCode());
        projectInstService.save(projectInst);
        return project;
    }

    /**
     * 申报新项目时 保存项目信息和其它相关联的信息
     * @param projectDto
     * @param instanceId
     * @param employeeCode
     * @return
     */
    public Project saveProjectNewVersion(ProjectDTO projectDto, String instanceId,
                                         String employeeCode) {
        //流程启动之后 入库项目 重要业务信息 用于列表查询 展示
        try {
            Project project = new Project();
            //为空 代表是新申报的
            if(Objects.isNull(projectDto.getId())){
                BeanUtils.copyProperties(projectDto, project);
                project.setCreateOn(LocalDateTime.now());
                project.setUpdateOn(LocalDateTime.now());
                project.setStage(ProjectStatusEnum.NOT_APPROVED.getCode());
                project.setStatus(ProjectStatusEnum.UNDER_INTERNAL_AUDIT.getCode());
                project.setInstCode(instanceId);
                project.setSponsor(employeeCode);
                String projectCode = generateProjectCodeUtil.generateProjectCode(projectDto);
                project.setProjectCode(projectCode);
                projectService.save(project);
            }else{
                //否则是重新提交的 新生成一个新版本的项目
                project = newProjectWithVersion(projectDto);
                if(Objects.nonNull(project)){
                    project.setInstCode(instanceId);
                    project.setSponsor(employeeCode);
                    projectService.updateById(project);
                }
            }

            //保存项目应用
            Boolean isApp = Objects.nonNull(projectDto.getIncludeApplication()) && CommonEnum.YES.getCode().equals(projectDto.getIncludeApplication())
                    ? Boolean.TRUE : Boolean.FALSE;
            //采取批量删除 批量添加的方式
            projectApplicationService.remove(Wrappers.lambdaQuery(ProjectApplication.class)
                    .eq(ProjectApplication::getProjectId,project.getId()));
            if (isApp && CollUtil.isNotEmpty(projectDto.getApplicationList())) {
                Project finalProject = project;
                List<ProjectApplication> applications = projectDto.getApplicationList().stream().map(application -> {
                    ProjectApplication projectApplication = new ProjectApplication();
                    BeanUtils.copyProperties(application, projectApplication);
                    projectApplication.setId(null);
                    projectApplication.setProjectId(finalProject.getId());
                    projectApplication.setBuildOrgCode(finalProject.getBuildOrgCode());
                    projectApplication.setBuildOrgName(finalProject.getBuildOrgName());
                    return projectApplication;
                }).collect(Collectors.toList());
                projectApplicationService.saveOrUpdateBatch(applications);
            }
            return project;
        } catch (Exception e) {
            log.error("项目信息入库错误 " + e);
            throw new BusinessException("项目信息入库错误 :" + e);
        }
    }

    /**
     * 在其它项目阶段 保存项目信息和其它相关联的信息
     * @param projectDto
     * @return
     */
    public Project reSaveProjectNewVersion(ProjectDTO projectDto) {
        //流程启动之后 入库项目 重要业务信息 用于列表查询 展示
        try {
            Project project = newProjectWithVersion(projectDto);

            //保存项目应用
            Boolean isApp = Objects.nonNull(projectDto.getIncludeApplication()) && CommonEnum.YES.getCode().equals(projectDto.getIncludeApplication())
                    ? Boolean.TRUE : Boolean.FALSE;
            //采取批量删除 批量添加的方式
            projectApplicationService.remove(Wrappers.lambdaQuery(ProjectApplication.class)
                    .eq(ProjectApplication::getProjectId,project.getId()));
            if (isApp && CollUtil.isNotEmpty(projectDto.getApplicationList())) {
                Project finalProject = project;
                List<ProjectApplication> applications = projectDto.getApplicationList().stream().map(application -> {
                    ProjectApplication projectApplication = new ProjectApplication();
                    BeanUtils.copyProperties(application, projectApplication);
                    projectApplication.setId(null);
                    projectApplication.setProjectId(finalProject.getId());
                    return projectApplication;
                }).collect(Collectors.toList());
                projectApplicationService.saveOrUpdateBatch(applications);
            }
            return project;
        } catch (Exception e) {
            log.error("项目信息入库错误 " + e);
            throw new BusinessException("项目信息入库错误 :" + e);
        }
    }

    /**
     * 重新提交工作流时 舍弃在原有项目修改
     * 新增一个新的项目 新的版本号
     */
    public Project newProjectWithVersion(ProjectDTO projecDto){
        Project oldProject = projectService.getById(projecDto.getId());
        Project project = new Project();
        VUtils.isTrue(Objects.isNull(oldProject))
                        .throwMessage("项目不存在!");
        BeanUtil.copyProperties(oldProject,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        BeanUtil.copyProperties(projecDto,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        stateMachineUtils.pass(project);
        projectService.save(project);

        projectService.update(Wrappers.lambdaUpdate(Project.class)
                .set(Project::getNewest,Boolean.FALSE)
                .ne(Project::getId,project.getId())
                .eq(Project::getProjectCode,project.getProjectCode()));

        return project;
    }

    public Project saveProjectWithVersionAndStatus(ProjectDTO projecDto,Integer stageCode,Integer statusCode){
        Project oldProject = projectService.getById(projecDto.getId());
        Project project = new Project();
        VUtils.isTrue(Objects.isNull(oldProject))
                .throwMessage("项目不存在!");
        BeanUtil.copyProperties(oldProject,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        BeanUtil.copyProperties(projecDto,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        if(Objects.nonNull(stageCode)){
            project.setStage(stageCode);
        }
        if(Objects.nonNull(statusCode)){
            project.setStatus(statusCode);
        }

        if(projectService.save(project)){
            projectService.update(Wrappers.lambdaUpdate(Project.class)
                    .set(Project::getNewest,Boolean.FALSE)
                    .ne(Project::getId,project.getId())
                    .eq(Project::getProjectCode,project.getProjectCode()));

            //app
            List<ProjectApplicationDTO> applicationList = projecDto.getApplicationList();
            if(CollUtil.isNotEmpty(applicationList)){
                List<ProjectApplication> apps = applicationList.stream()
                        .map(a -> {
                            ProjectApplication app = BeanUtil.copyProperties(a,ProjectApplication.class);
                            app.setId(null);
                            app.setProjectId(project.getId());
                            app.setBuildOrgName(project.getBuildOrgName());
                            app.setBuildOrgCode(project.getBuildOrgCode());
                            return app;
                        })
                        .collect(Collectors.toList());
                projectApplicationService.saveBatch(apps);
            }
        }

        return project;
    }

    public Project saveProjectWithVersion(ProjectDTO projecDto,String instanceId,Integer instType){
        Project oldProject = projectService.getById(projecDto.getId());
        Project project = new Project();
        VUtils.isTrue(Objects.isNull(oldProject))
                .throwMessage("项目不存在!");
        BeanUtil.copyProperties(oldProject,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        BeanUtil.copyProperties(projecDto,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());

        if(projectService.save(project)){
            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setCreatOn(LocalDateTime.now());
            projectInst.setUpdateOn(LocalDateTime.now());
            projectInst.setInstType(instType);
            projectInstService.save(projectInst);

            projectService.update(Wrappers.lambdaUpdate(Project.class)
                    .set(Project::getNewest,Boolean.FALSE)
                    .ne(Project::getId,project.getId())
                    .eq(Project::getProjectCode,project.getProjectCode()));

            //app
            List<ProjectApplicationDTO> applicationList = projecDto.getApplicationList();
            if(CollUtil.isNotEmpty(applicationList)){
                List<ProjectApplication> apps = applicationList.stream()
                        .map(a -> {
                            ProjectApplication app = BeanUtil.copyProperties(a,ProjectApplication.class);
                            app.setId(null);
                            app.setProjectId(project.getId());
                            app.setBuildOrgName(project.getBuildOrgName());
                            app.setBuildOrgCode(project.getBuildOrgCode());
                            return app;
                        })
                        .collect(Collectors.toList());
                projectApplicationService.saveBatch(apps);
            }
        }

        return project;
    }
    public Project saveProjectWithVersion(Project oldProject,String instanceId,Integer instType){
        Project project = new Project();
        VUtils.isTrue(Objects.isNull(oldProject))
                .throwMessage("项目不存在!");
        BeanUtil.copyProperties(oldProject,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());

        if(projectService.save(project)){
            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setCreatOn(LocalDateTime.now());
            projectInst.setUpdateOn(LocalDateTime.now());
            projectInst.setInstType(instType);
            projectInstService.save(projectInst);

            projectService.update(Wrappers.lambdaUpdate(Project.class)
                    .set(Project::getNewest,Boolean.FALSE)
                    .ne(Project::getId,project.getId())
                    .eq(Project::getProjectCode,project.getProjectCode()));

            //app
            List<ProjectApplication> applicationList = projectApplicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                    .eq(ProjectApplication::getProjectId,oldProject.getId()));
            if(CollUtil.isNotEmpty(applicationList)){
                List<ProjectApplication> apps = applicationList.stream()
                        .map(a -> {
                            ProjectApplication app = BeanUtil.copyProperties(a,ProjectApplication.class);
                            app.setId(null);
                            app.setProjectId(project.getId());
                            app.setBuildOrgName(project.getBuildOrgName());
                            app.setBuildOrgCode(project.getBuildOrgCode());
                            return app;
                        })
                        .collect(Collectors.toList());
                projectApplicationService.saveBatch(apps);
            }
        }

        return project;
    }
    public Project saveProjectWithVersionAndPass(Project oldProject,String instanceId,Integer instType){
        Project project = new Project();
        VUtils.isTrue(Objects.isNull(oldProject))
                .throwMessage("项目不存在!");
        BeanUtil.copyProperties(oldProject,project, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        project.setVersion(oldProject.getVersion() + 1);
        project.setId(null);
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        stateMachineUtils.pass(project);

        if(projectService.save(project)){
            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setCreatOn(LocalDateTime.now());
            projectInst.setUpdateOn(LocalDateTime.now());
            projectInst.setInstType(instType);
            projectInstService.save(projectInst);

            projectService.update(Wrappers.lambdaUpdate(Project.class)
                    .set(Project::getNewest,Boolean.FALSE)
                    .ne(Project::getId,project.getId())
                    .eq(Project::getProjectCode,project.getProjectCode()));

            //app
            List<ProjectApplication> applicationList = projectApplicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                    .eq(ProjectApplication::getProjectId,oldProject.getId()));
            if(CollUtil.isNotEmpty(applicationList)){
                List<ProjectApplication> apps = applicationList.stream()
                        .map(a -> {
                            ProjectApplication app = BeanUtil.copyProperties(a,ProjectApplication.class);
                            app.setId(null);
                            app.setProjectId(project.getId());
                            app.setBuildOrgName(project.getBuildOrgName());
                            app.setBuildOrgCode(project.getBuildOrgCode());
                            return app;
                        })
                        .collect(Collectors.toList());
                projectApplicationService.saveBatch(apps);
            }
        }

        return project;
    }

    /**
     * @param projectId 项目详情
     * @return com.ningdatech.pmapi.projectlib.model.entity.vo.ProjectDetailVO
     * @author ZPF
     * @since 2023/02/11 11:15
     */
    public ProjectDetailVO getProjectDetail(Long projectId) {
        // 查询项目申报信息
        Project projectInfo = projectService.getNewProject(projectId);
        if (Objects.isNull(projectInfo)) {
            return null;
        }
        ProjectDetailVO vo = new ProjectDetailVO();
        BeanUtils.copyProperties(projectInfo, vo);
        vo.buildDynamicForm(projectInfo.getDynamicForm());
        // 查询应用
        List<ProjectApplication> applications = applicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(ProjectApplication::getProjectId, vo.getId()));
        Optional.ofNullable(applications).ifPresent(apps ->
                vo.setProjectApplications(CollUtils.convert(apps,
                        ProjectHelper::convertVO)
                ));

        List<Long> allVersionProjectId = projectService.getAllVersionProjectId(projectInfo);
        //查询采购备案
        Purchase purchase = purchaseService.getOne(Wrappers.lambdaQuery(Purchase.class)
                .in(Purchase::getProjectId, allVersionProjectId)
                .last(BizConst.LIMIT_1));
        vo.setPurchase(BeanUtil.copyProperties(purchase, PurchaseVO.class));

        //查询合同备案
        Contract contract = contractService.getOne(Wrappers.lambdaQuery(Contract.class)
                .in(Contract::getProjectId, allVersionProjectId)
                .last(BizConst.LIMIT_1));
        ContractVO contractVO = BeanUtil.copyProperties(contract, ContractVO.class);
        vo.setContract(contractVO);
        List<PaymentPlan> payments = paymentPlanService.list(Wrappers.lambdaQuery(PaymentPlan.class)
                .in(PaymentPlan::getProjectId, allVersionProjectId)
                .orderByAsc(PaymentPlan::getPaymentTime));
        if(Objects.nonNull(contractVO)){
            contractVO.setPayments(convertPayments(payments,
                    contractVO.getTotalAmount()));
        }


        //查询初验信息
        List<PreInsAcceptancePerson> acceptancePersons = acceptancePersonService.list(Wrappers.lambdaQuery(PreInsAcceptancePerson.class)
                .in(PreInsAcceptancePerson::getProjectId, allVersionProjectId)
                .orderByAsc(PreInsAcceptancePerson::getCreateOn));
        vo.setAcceptancePersons(convertPersons(acceptancePersons));

        //查询终验信息
        //查询年度投资金额 要是已验收的项目
        if (ProjectStatusEnum.ACCEPTED.getCode().equals(projectInfo.getStatus())) {
            List<AnnualAmountVO> annualAmounts = Lists.newArrayList();
            annualAmounts.add(AnnualAmountVO.builder()
                    .projectId(projectInfo.getId())
                    .projectYear(projectInfo.getProjectYear())
                    .annualAmount(projectInfo.getAnnualPlanAmount())
                    .build());
            List<ProjectRenewalFundDeclaration> renewalDeclarations = renewalFundDeclarationService.list(Wrappers.lambdaQuery(ProjectRenewalFundDeclaration.class)
                    .eq(ProjectRenewalFundDeclaration::getProjectId, vo.getId())
                    .orderByAsc(ProjectRenewalFundDeclaration::getProjectYear));
            Optional.ofNullable(renewalDeclarations).ifPresent(declarations ->
                    declarations.forEach(declaration -> {
                        annualAmounts.add(AnnualAmountVO.builder()
                                .projectId(projectInfo.getId())
                                .projectYear(declaration.getProjectYear())
                                .annualAmount(declaration.getAnnualPaymentAmount())
                                .build());
                    })
            );
            vo.setAnnualAccumulateAmountList(annualAmounts);
        }
        // 处理文件名
        List<Long> fileIdList = new ArrayList<>();
        BizUtils.notBlank(vo.getApprovedFile(), w -> fileIdList.add(Long.parseLong(w)));
        BizUtils.notBlank(vo.getApprovedConstructionPlanFile(), w -> fileIdList.add(Long.parseLong(w)));
        if (!fileIdList.isEmpty()) {
            List<File> files = fileService.listByIds(fileIdList);
            Map<String, String> fileMap = CollUtils.listToMap(files, w -> w.getId().toString(), File::getOriginalFileName);
            vo.setApprovedFileName(fileMap.get(vo.getApprovedFile()));
            vo.setApprovedConstructionPlanFileName(fileMap.get(vo.getApprovedConstructionPlanFile()));
        }
        return vo;
    }

    public List<ProcessDetailVO> processScheduleDetail(Long projectId) {
        List<ProcessDetailVO> processDetailVOS = new ArrayList<>();
        return processExecuteHandle.handle(projectId, processDetailVOS);
    }

    private boolean checkCanRead(Set<String> subOrgSet, Project project) {
        // 从申请借阅信息表中查出本单位及下属单位审批通过的项目
        HashSet<Long> borrowProjectIdSet = new HashSet<>();
        List<Long> applyBorrowProjectIdList = projectApplyBorrowService
                .list(Wrappers.lambdaQuery(ProjectApplyBorrow.class)
                        .in(ProjectApplyBorrow::getApplyBorrowEmpPosUnitCode, subOrgSet)
                        .eq(ProjectApplyBorrow::getIsSuccess, Boolean.TRUE))
                .stream().map(ProjectApplyBorrow::getProjectId).filter(borrowProjectIdSet::add).collect(Collectors.toList());
        return applyBorrowProjectIdList.contains(project.getId());
    }

    private Set<String> getSubOrgList(String empPosUnitCode) {
        HashSet<String> orgSet = new HashSet<>();
        // 先将自己加入子集
        orgSet.add(empPosUnitCode);
        Set<String> subOrgList = dingOrganizationService
                .list(Wrappers.lambdaQuery(DingOrganization.class).eq(DingOrganization::getParentCode, empPosUnitCode))
                .stream().map(DingOrganization::getOrganizationCode).collect(Collectors.toSet());
        // 遍历子集
        for (String orgCode : subOrgList) {
            orgSet.addAll(getSubOrgList(orgCode));
        }
        return orgSet;
    }


    public void exportList(ProjectListReq param, HttpServletResponse response) {
        param.setPageNumber(CommonConst.EXPORT_PAGE_NUMBER);
        param.setPageSize(CommonConst.EXPORT_PAGE_SIZE);
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(param);
        query.eq(Project::getNewest,Boolean.TRUE);
        List<Project> projects = projectService.list(query);

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = "项目库列表";
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getProjectExportDatas(param.getExportOptionList(), projects);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);
    }

    /**
     * build 项目的角色权限到req
     *
     * @param query
     */
    public UserFullInfoDTO buildProjectLibPermission(LambdaQueryWrapper<Project> query,UserFullInfoDTO user) {
        Optional<DataScopeDTO> currentUserDataScope = DataScopeUtil.getCurrentUserDataScope(user);
        if (!currentUserDataScope.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            return user;
        }

        switch (currentUserDataScope.get().getRole()) {
            case NORMAL_MEMBER:
                //普通用户 只能看到自己单位去申报的
                query.eq(Project::getBuildOrgCode, user.getEmpPosUnitCode());
                break;
            case COMPANY_MANAGER:
                //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                query.and(q1 -> q1.eq(Project::getBuildOrgCode, user.getEmpPosUnitCode()).or(q2 ->
                        q2.eq(Project::getStage, ProjectStatusEnum.NOT_APPROVED.getCode())
                                .eq(Project::getStatus, ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode())
                                .eq(Project::getSuperOrgCode, user.getEmpPosUnitCode())));
                break;
            case SUPER_ADMIN:
                //超级管理员 看到丽水全市的 并且也要判断他 同时是不是单位管理员
                break;
            case REGION_MANAGER:
                //区域管理员 看到自己区域的项目 如果是市本级 就看全市的
                if (RegionConst.RC_LS.equals(user.getRegionCode())) {
                    Collection<String> regionCodes = regionCacheHelper.listChildRegionCodeList(user.getRegionCode(),
                            RegionConst.RL_CITY);
                    query.in(Project::getAreaCode, regionCodes);
                } else {
                    query.eq(Project::getAreaCode, user.getRegionCode());
                }
                break;
            case VISITOR:
                //访客可以看全市的
                break;
            case DASHBOARD:
                break;
            default:
                //没有权限的话 就让它查不到
                query.eq(Project::getId, 0L);
                break;
        }
        return user;
    }

    public void saveProjectByApplyDelay(DelayedApplyDTO dto, Project project, String instanceId) {

        // 更新项目流程实例ID
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        projectService.updateById(project);

        // 保存延期申请信息
        ProjectDelayApply projectDelayApply = new ProjectDelayApply();
        projectDelayApply.setDelayTime(dto.getDelayedMonth());
        projectDelayApply.setDelayApplyFile(dto.getSupportingMaterials());
        projectDelayApply.setDelayApplyReason(dto.getDelayedReason());
        projectDelayApply.setProjectId(project.getId());
        projectDelayApply.setInstanceId(instanceId);
        projectDelayApply.setCreateBy(LoginUserUtil.getUserId());
        projectDelayApply.setCreateOn(LocalDateTime.now());
        projectDelayApply.setUpdateBy(LoginUserUtil.getUserId());
        projectDelayApply.setUpdateOn(LocalDateTime.now());
        projectDelayApplyService.save(projectDelayApply);

        //保存项目和实例的关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setProjectId(project.getId());
        projectInst.setInstCode(instanceId);
        projectInst.setInstType(InstTypeEnum.APPLY_DELAY.getCode());
        projectInst.setCreatOn(LocalDateTime.now());
        projectInst.setUpdateOn(LocalDateTime.now());
        projectInstService.save(projectInst);
    }

    public void saveProjectByApplyBorrow(Project project, UserFullInfoDTO user, String instanceId) {
        // 更新项目流程实例ID
        project.setInstCode(instanceId);
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        projectService.updateById(project);

        // 保存项目借阅信息
        ProjectApplyBorrow projectApplyBorrow = new ProjectApplyBorrow();
        projectApplyBorrow.setProjectId(project.getId());
        projectApplyBorrow.setApplyBorrowEmployeeCode(user.getEmployeeCode());
        projectApplyBorrow.setApplyBorrowEmpPosUnitCode(user.getEmpPosUnitCode());
        projectApplyBorrow.setInstanceId(instanceId);
        projectApplyBorrow.setCreateOn(LocalDateTime.now());
        projectApplyBorrow.setCreateBy(LoginUserUtil.getUserId());
        projectApplyBorrow.setUpdateOn(LocalDateTime.now());
        projectApplyBorrow.setUpdateBy(LoginUserUtil.getUserId());
        projectApplyBorrow.setIsSuccess(Boolean.FALSE);
        projectApplyBorrowService.save(projectApplyBorrow);

        //保存项目和实例的关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setProjectId(project.getId());
        projectInst.setInstCode(instanceId);
        projectInst.setInstType(InstTypeEnum.APPLY_BORROW.getCode());
        projectInst.setCreatOn(LocalDateTime.now());
        projectInst.setUpdateOn(LocalDateTime.now());
        projectInstService.save(projectInst);
    }

    /**
     * 应用列表
     * @param req
     * @return
     */
    public PageVo<ProjectApplicationVO> applicationList(ProjectApplicationListReq req) {
        Page<ProjectApplication> page = req.page();
        LambdaQueryWrapper<ProjectApplication> wrapper = Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(StringUtils.isNotBlank(req.getBuildOrgName()),ProjectApplication::getBuildOrgName,req.getBuildOrgName());

        if(StringUtils.isNotBlank(req.getApplicationName())){
            wrapper.and(q1 -> q1.like(ProjectApplication::getApplicationName,req.getApplicationName())
                    .or(q2 -> q2.like(ProjectApplication::getRelatedExistsApplication,req.getApplicationName())));
        }
        wrapper.orderByDesc(ProjectApplication::getUpdateOn);
        projectApplicationService.page(page,wrapper);

        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<ProjectApplicationVO> res = page.getRecords().stream()
                .map(app -> BeanUtil.copyProperties(app,ProjectApplicationVO.class))
                .collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    private List<PreInsAcceptancePersonVO> convertPersons(List<PreInsAcceptancePerson> acceptancePersons) {
        if(CollUtil.isEmpty(acceptancePersons)){
            return Collections.emptyList();
        }

        return acceptancePersons.stream().map(u -> BeanUtil.copyProperties(u,PreInsAcceptancePersonVO.class)).collect(Collectors.toList());
    }

    private List<PaymentPlanVO> convertPayments(List<PaymentPlan> payments, BigDecimal totalAmount) {
        if(CollUtil.isEmpty(payments)){
            return Collections.emptyList();
        }
        return payments.stream().map(p -> {
                    PaymentPlanVO vo = BeanUtil.copyProperties(p, PaymentPlanVO.class);
                    vo.setRatio((Objects.isNull(totalAmount)||totalAmount.compareTo(BigDecimal.ZERO) == 0) ? "0%"
                            : p.getPaymentAmount().multiply(BigDecimal.valueOf(100))
                            .divide(totalAmount,BigDecimal.ROUND_CEILING,BigDecimal.ROUND_CEILING) + "%");
                    return vo;
                })
                .collect(Collectors.toList());
    }

    /**
     * 历史版本 项目列表
     * @param id
     * @return
     */
    public List<ProjectDetailVO> historyProjects(Long id) {
        Project project = projectService.getById(id);

        if(Objects.isNull(project)){
            return Collections.emptyList();
        }

        List<Project> historyProjects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectCode, project.getProjectCode())
                .orderByDesc(Project::getCreateOn));

        if(CollUtil.isEmpty(historyProjects)){
            return Collections.emptyList();
        }

        return historyProjects.stream()
                .map(p -> BeanUtil.copyProperties(p,ProjectDetailVO.class))
                .collect(Collectors.toList());
    }
}
