package com.ningdatech.pmapi.projectlib.model.vo;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.ningdatech.pmapi.common.enumeration.CommonEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectTypeEnum;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Builder;
import lombok.Data;
import lombok.experimental.Tolerate;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * ProjectLibListItemVO
 * </p>
 *
 * @author WendyYang
 * @since 15:13 2023/2/1
 */
@Data
@Builder
@ApiModel("项目库列表视图")
public class ProjectLibListItemVO {

    @Tolerate
    public ProjectLibListItemVO() {
    }

    @ApiModelProperty("项目ID")
    private Long id;

    @ApiModelProperty("项目名称")
    private String projectName;

    @ApiModelProperty("申报金额")
    private BigDecimal declaredAmount;

    @ApiModelProperty("批复金额")
    private BigDecimal approvedAmount;

    @ApiModelProperty("立项批复时间")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime approvalDate;

    @ApiModelProperty("项目类型")
    private Integer projectType;

    @ApiModelProperty("项目阶段")
    private Integer stage;

    @ApiModelProperty("项目状态")
    private Integer status;

    @ApiModelProperty("申报年度")
    private Integer projectYear;

    @ApiModelProperty("申报单位")
    private String buildOrg;

    @ApiModelProperty("业务领域")
    private Integer bizDomain;

    @ApiModelProperty("流程状态")
    private Integer processStatus;

    @ApiModelProperty("实例code")
    private String instCode;

    @ApiModelProperty("建设单位名称")
    private String buildOrgName;

    @ApiModelProperty("建设单位统一社会信用代码")
    private String buildOrgCode;

    private String projectTypeName;

    @ApiModelProperty("成交金额")
    private BigDecimal transactionAmount;

    @ApiModelProperty("成交时间")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime transactionTime;

    @ApiModelProperty("交货时间")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime deliveryTime;

    @ApiModelProperty("合同总金额")
    private BigDecimal contractAmount;

    @ApiModelProperty("年度支付计划-年度支付计划(元)")
    private BigDecimal annualPlanAmount;

    @ApiModelProperty("建设周期 月")
    private Integer buildCycle;

    @ApiModelProperty("计划验收时间")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime planAcceptanceTime;

    public String getProjectTypeName() {
        if (Objects.nonNull(this.projectType)) {
            Optional.ofNullable(ProjectTypeEnum.getDesc(this.projectType))
                    .ifPresent(desc -> this.projectTypeName = desc);
        }
        return this.projectTypeName;
    }

    @ApiModelProperty("创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private LocalDateTime createOn;

    @ApiModelProperty("修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private LocalDateTime updateOn;

    @ApiModelProperty("能否被预审申报")
    private Boolean canPreDeclared = Boolean.FALSE;

    @ApiModelProperty("21位项目编号")
    private String projectCode;

    @ApiModelProperty("是否有上级条线主管部门 0没有 1有")
    private Integer isHigherSuperOrg;

    @ApiModelProperty("是否可以申请延期")
    private Boolean canDelayApply = Boolean.TRUE;

    //预审申报时候 需不需要上传上级条线意见文件
    private Boolean needUploadSuperLineFile;
    public Boolean getNeedUploadSuperLineFile() {
        UserInfoDetails userInfoDetail = LoginUserUtil.loginUserDetail();
        //要满足条件 1.当前登录人是市本级单位 2.有上级条线单位 3.项目状态是待预审
        if (Objects.nonNull(userInfoDetail) && userInfoDetail.getIsMunicipalOrg()
                && Objects.nonNull(this.isHigherSuperOrg)
                && ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode().equals(this.status)
                && CommonEnum.YES.getCode().equals(this.isHigherSuperOrg)) {
            this.needUploadSuperLineFile = Boolean.TRUE;
            return this.needUploadSuperLineFile;
        }
        this.needUploadSuperLineFile = null;
        return this.needUploadSuperLineFile;
    }
}
