package com.ningdatech.pmapi.sys.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.IdVo;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.file.entity.vo.result.AttachFileVo;
import com.ningdatech.file.service.FileService;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.util.BizUtils;
import com.ningdatech.pmapi.organization.model.entity.DingEmployeeInfo;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingEmployeeInfoService;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.staging.enums.MsgTypeEnum;
import com.ningdatech.pmapi.staging.service.INdWorkNoticeStagingService;
import com.ningdatech.pmapi.sys.enumeration.NoticeTypeEnum;
import com.ningdatech.pmapi.sys.model.entity.Notice;
import com.ningdatech.pmapi.sys.model.entity.Notify;
import com.ningdatech.pmapi.sys.model.req.NoticeListReq;
import com.ningdatech.pmapi.sys.model.req.NoticeSaveReq;
import com.ningdatech.pmapi.sys.model.req.NoticeStatusModifyReq;
import com.ningdatech.pmapi.sys.model.vo.NoticeDetailVO;
import com.ningdatech.pmapi.sys.model.vo.NoticeListItemVO;
import com.ningdatech.pmapi.sys.service.INoticeService;
import com.ningdatech.pmapi.sys.service.INotifyService;
import com.ningdatech.pmapi.todocenter.bean.entity.WorkNoticeInfo;
import com.ningdatech.pmapi.todocenter.model.dto.ProjectAuditMsgExtraDTO;
import com.ningdatech.pmapi.user.entity.UserInfo;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.runtime.ActivityInstance;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.ningdatech.pmapi.todocenter.constant.WorkNoticeContant.AUDIT_WORK_TITLE;

/**
 * <p>
 * MsgManage
 * </p>
 *
 * @author WendyYang
 * @since 21:30 2022/7/21
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class NoticeManage {

    private final INoticeService noticeService;
    private final FileService fileService;

    private final IUserInfoService userInfoService;

    private final IDingEmployeeInfoService dingEmployeeInfoService;

    private final IDingOrganizationService dingOrganizationService;

    private final INotifyService notifyService;

    private final INdWorkNoticeStagingService workNoticeStagingService;

    private final RuntimeService runtimeService;

    private final UserInfoHelper userInfoHelper;

    @Transactional(rollbackFor = Exception.class)
    public IdVo<Long> saveOrModify(NoticeSaveReq req) {
        Notice notice = BeanUtil.copyProperties(req, Notice.class);
        noticeService.saveOrUpdate(notice);
        return IdVo.of(notice.getId());
    }

    public NoticeDetailVO detail(Long id) {
        Notice notice = noticeService.getById(id);
        NoticeDetailVO detail = BeanUtil.copyProperties(notice, NoticeDetailVO.class);
        List<Long> fileIds = BizUtils.splitToLong(notice.getAttachment());
        if (!fileIds.isEmpty()) {
            List<AttachFileVo> attachFiles = fileService.getByIds(fileIds);
            detail.setAttachments(attachFiles);
        }
        return detail;
    }

    public boolean changeEnabled(NoticeStatusModifyReq req) {
        Notice notice = new Notice();
        notice.setId(req.getId());
        notice.setEnabled(req.getStatus());
        return noticeService.updateById(notice);
    }

    public void topped(Long id) {
        Notice notice = new Notice();
        notice.setId(id);
        notice.setToppedTime(LocalDateTime.now());
        noticeService.updateById(notice);
    }


    public PageVo<NoticeListItemVO> dashboardList(Integer limit, Integer type) {
        NoticeListReq req = new NoticeListReq();
        req.setPageSize(limit);
        req.setEnabled(true);
        req.setType(type);
        return listByManager(req);
    }

    public PageVo<NoticeListItemVO> listByManager(NoticeListReq req) {
        LambdaQueryWrapper<Notice> wrapper = Wrappers.lambdaQuery(Notice.class)
                .eq(req.getEnabled() != null, Notice::getEnabled, req.getEnabled())
                .like(StrUtil.isNotBlank(req.getTitle()), Notice::getTitle, req.getTitle())
                .eq(req.getType() != null, Notice::getType, req.getType())
                .orderByDesc(Notice::getToppedTime, Notice::getUpdateOn);
        Page<Notice> page = noticeService.page(req.page(), wrapper);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<NoticeListItemVO> tempDataList = CollUtils.convert(page.getRecords(), w -> NoticeListItemVO
                .builder()
                .id(w.getId())
                .type(w.getType())
                .title(w.getTitle())
                .enabled(w.getEnabled())
                .createOn(w.getCreateOn())
                .topped(w.getToppedTime() != null)
                .build());
        return PageVo.of(tempDataList, page.getTotal());
    }

    public Map<String,List<NoticeListItemVO>> listToMapByManager(NoticeListReq req) {
        LambdaQueryWrapper<Notice> wrapper = Wrappers.lambdaQuery(Notice.class)
                .eq(req.getEnabled() != null, Notice::getEnabled, req.getEnabled())
                .like(StrUtil.isNotBlank(req.getTitle()), Notice::getTitle, req.getTitle())
                .eq(req.getType() != null, Notice::getType, req.getType())
                .orderByDesc(Notice::getToppedTime, Notice::getUpdateOn);
        List<Notice> records = noticeService.list(wrapper);
        return CollUtils.convert(records, w -> NoticeListItemVO
                .builder()
                .id(w.getId())
                .type(w.getType())
                .title(w.getTitle())
                .enabled(w.getEnabled())
                .createOn(w.getCreateOn())
                .topped(w.getToppedTime() != null)
                .build()).stream().collect(Collectors.groupingBy(v -> NoticeTypeEnum.getNameByCode(v.getType())));
    }

    public void delNotice(Long id) {
        noticeService.removeById(id);
    }


    /**
     * 发送消息
     * @param employeeCode 员工号
     * @param userId 用户ID
     * @param project 项目
     * @param procDefinitionName 流程定义名
     * @param template 消息模板内容
     * @param msgTypeEnum 枚举
     */
    public void sendNotice(String employeeCode, Long userId, Project project, String procDefinitionName,
                           String template, MsgTypeEnum msgTypeEnum){
        // 获取发送浙政钉工作通知必要信息
        WorkNoticeInfo passWorkNoticeInfo = getSendWorkNoticeInfo(employeeCode);
        String passMsg = String.format(template, project.getProjectName(), procDefinitionName);
        passWorkNoticeInfo.setMsg(passMsg);
        // 放入系统通知表中，保存记录
        Notify notify = assemblyAuditNotify(userId, project, passMsg);
        notify.setType(msgTypeEnum.name());
        notifyService.save(notify);
        // 放入工作通知暂存表中，通过扫表异步发送
        workNoticeStagingService.addByWorkNotice(passWorkNoticeInfo, msgTypeEnum);
    }

    /**
     * 获取发送浙政钉工作通知的信息
     *
     * @param currentEmployeeCode
     * @return com.ningdatech.pmapi.todocenter.bean.entity.WorkNoticeInfo
     * @author CMM
     * @since 2023/02/15 14:04
     */
    public WorkNoticeInfo getSendWorkNoticeInfo(String currentEmployeeCode) {
        UserInfo auditUserInfo = userInfoService.getOne(Wrappers.lambdaQuery(UserInfo.class).eq(UserInfo::getEmployeeCode,currentEmployeeCode).last("limit 1"));
        if (Objects.isNull(auditUserInfo)) {
            throw new BizException("该用户不存在!");
        }
        WorkNoticeInfo workNoticeInfo = new WorkNoticeInfo();
        Long accountId = auditUserInfo.getAccountId();
        if (Objects.isNull(accountId)) {
            throw new BizException("该用户没有录入浙政钉信息!");
        }
        workNoticeInfo.setAccountId(accountId);
        // 根据浙政钉用户ID获取部门code
        DingEmployeeInfo employeeInfo = dingEmployeeInfoService.getOne(Wrappers.lambdaQuery(DingEmployeeInfo.class)
                .eq(DingEmployeeInfo::getAccountId, accountId)
                .eq(DingEmployeeInfo::getMainJob,String.valueOf(Boolean.TRUE))
                .last("limit 1"));
        String organizationCode = employeeInfo.getOrganizationCode();
        workNoticeInfo.setOrganizationCode(organizationCode);
        // 根据部门code获取部门名称
        DingOrganization dingOrganization = dingOrganizationService.getOne(Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getOrganizationCode, organizationCode));
        String organizationName = dingOrganization.getOrganizationName();
        workNoticeInfo.setOrganizationName(organizationName);
        // 构建唯一的消息ID
        String bizMsgId = "ZD_WORK_NOTICE_" + StrUtil.UNDERLINE + organizationCode + StrUtil.UNDERLINE
                + organizationName + accountId + StrUtil.UNDERLINE + System.currentTimeMillis();
        workNoticeInfo.setBizMsgId(bizMsgId);
        String receiverUserId = String.valueOf(accountId);
        workNoticeInfo.setReceiverUserId(receiverUserId);
        return workNoticeInfo;
    }

    /**
     * 装配项目审核工作通知
     * @param userId
     * @param project
     * @param msg
     */
    private Notify assemblyAuditNotify(Long userId, Project project, String msg) {
        Notify notify = new Notify();
        notify.setTitle(AUDIT_WORK_TITLE);
        notify.setUserId(userId);
        notify.setContent(msg);
        notify.setReaded(Boolean.FALSE);
        notify.setCreateTime(LocalDateTime.now());
        ProjectAuditMsgExtraDTO msgExtraDto = new ProjectAuditMsgExtraDTO();
        msgExtraDto.setProjectId(project.getId());
        msgExtraDto.setInstanceId(project.getInstCode());
        String extraJson = JSON.toJSONString(msgExtraDto);
        notify.setExtraInfo(extraJson);
        return notify;
    }

    /**
     * 找寻第一个审核人 去发消息
     * @param project
     * @param formName
     * @param template
     * @param msgTypeEnum
     */
    public void sendFirtUser(Project project, String formName,String instanceId, String template, MsgTypeEnum msgTypeEnum) {
        try {
            List<ActivityInstance> activityInstances = runtimeService.createActivityInstanceQuery()
                    .processInstanceId(instanceId)
                    .activityType("userTask")
                    .orderByActivityInstanceStartTime()
                    .asc()
                    .list();

            for (ActivityInstance activityInstance : activityInstances) {
                if(StringUtils.isNotBlank(activityInstance.getAssignee())){
                    UserFullInfoDTO user = userInfoHelper.getUserFullInfoByEmployeeCode(activityInstance.getAssignee());
                    sendNotice(activityInstance.getAssignee(), user.getUserId(), project, formName,
                            template, msgTypeEnum);
                }
            }
        }catch (Exception e){
            log.error("发送消息失败 :" + e);
        }

    }
}
