package com.ningdatech.pmapi.todocenter.manage;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.pmapi.projectlib.enumeration.*;
import com.ningdatech.pmapi.todocenter.constant.TodoCenterContant;
import com.ningdatech.pmapi.todocenter.handle.PassHandle;
import com.ningdatech.pmapi.todocenter.handle.WithDrawHandle;
import com.ningdatech.pmapi.todocenter.model.vo.TodoNumVO;
import com.wflow.contants.HisProInsEndActId;
import com.wflow.workflow.enums.ProcessStatusEnum;
import com.wflow.workflow.utils.ProcessTaskUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.FlowElementsContainer;
import org.flowable.bpmn.model.SubProcess;
import org.flowable.engine.HistoryService;
import org.flowable.engine.RepositoryService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.task.api.Task;
import org.flowable.task.api.history.HistoricTaskInstance;
import org.flowable.variable.api.history.HistoricVariableInstance;
import org.springframework.beans.BeanUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import com.ningdatech.file.service.FileService;
import com.ningdatech.pmapi.common.constant.CommonConst;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.model.entity.ExcelExportWriter;
import com.ningdatech.pmapi.common.util.ExcelDownUtil;
import com.ningdatech.pmapi.irs.sign.IRSAPIRequest;
import com.ningdatech.pmapi.projectdeclared.manage.DefaultDeclaredProjectManage;
import com.ningdatech.pmapi.projectlib.manage.ProjectLibManage;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectApplication;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectInst;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.projectlib.service.IProjectApplicationService;
import com.ningdatech.pmapi.projectlib.service.IProjectInstService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.signature.entity.CompanySignature;
import com.ningdatech.pmapi.signature.service.ICompanySignatureService;
import com.ningdatech.pmapi.todocenter.enumeration.IsAppendProjectEnum;
import com.ningdatech.pmapi.todocenter.enumeration.IsOrNotEnum;
import com.ningdatech.pmapi.todocenter.model.dto.AdjustHandleDTO;
import com.ningdatech.pmapi.todocenter.model.dto.PdfGenerateDTO;
import com.ningdatech.pmapi.todocenter.model.dto.SealInfoDTO;
import com.ningdatech.pmapi.todocenter.model.dto.SignReqDTO;
import com.ningdatech.pmapi.todocenter.model.po.TodoCenterStatisticsPO;
import com.ningdatech.pmapi.todocenter.model.req.ProcessDetailReq;
import com.ningdatech.pmapi.todocenter.model.req.ToBeProcessedExportReq;
import com.ningdatech.pmapi.todocenter.model.req.ToBeProcessedReq;
import com.ningdatech.pmapi.todocenter.model.vo.ProcessProgressDetailVo;
import com.ningdatech.pmapi.todocenter.model.vo.ResToBeProcessedVO;
import com.ningdatech.pmapi.todocenter.model.vo.TodoCenterStatisticsVO;
import com.ningdatech.pmapi.todocenter.service.StatisticsService;
import com.ningdatech.pmapi.todocenter.utils.BuildUserUtils;
import com.ningdatech.pmapi.todocenter.utils.PdfUtils;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.wflow.exception.BusinessException;
import com.wflow.workflow.bean.dto.ReqProcessHandlerDTO;
import com.wflow.workflow.bean.dto.TodoCenterListReqDTO;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.vo.ProcessInstanceVo;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.bean.vo.ProcessTaskVo;
import com.wflow.workflow.service.ProcessInstanceService;
import com.wflow.workflow.service.ProcessTaskService;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import sun.misc.BASE64Decoder;

/**
 * @author ZPF
 * @since 2023/02/19 16:09
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class TodoCenterManage {

    private final ProcessTaskService processTaskService;
    private final HistoryService historyService;
    private final TaskService taskService;
    private final IProjectService projectService;
    private final ProjectLibManage projectLibManage;
    private final ProcessInstanceService processInstanceService;
    private final DefaultDeclaredProjectManage defaultDeclaredProjectManage;
    private final IProjectApplicationService projectApplicationService;
    private final UserInfoHelper userInfoHelper;
    private final BuildUserUtils buildUserUtils;
    private final IProjectInstService projectInstService;
    private final PdfUtils pdfUtils;
    private final FileService fileService;
    private final ICompanySignatureService companySignatureService;
    private final StatisticsService statisticsService;

    private final HandlerManage handlerManage;

    private final WithDrawHandle withDrawHandle;

    private final PassHandle passHandle;

    private final RepositoryService repositoryService;

    /**
     * 待办中心待我处理项目列表查询
     *
     * @param param
     * @author zpf
     * @since 2023/02/18
     */
    public PageVo<ResToBeProcessedVO> todoProjectList(ToBeProcessedReq param) {
        String employeeCode = param.getEmployeeCode();
        if(StringUtils.isBlank(employeeCode)){
            // 获取登录用户ID
            Long userId = LoginUserUtil.getUserId();
            // 获取登录用户全量信息
            UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
            // 获取员工浙政钉code
            if(Objects.nonNull(userFullInfo)){
                employeeCode = userFullInfo.getEmployeeCode();
            }
            if(StringUtils.isBlank(employeeCode)){
                return PageVo.empty();
            }
        }

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        if(CollUtil.isEmpty(projects)){
            return PageVo.empty();
        }
        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());

        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());
        if(CollUtil.isEmpty(instCodes)){
            return PageVo.empty();
        }
        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        req.setInstCodes(instCodes);
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setEmployeeCode(employeeCode);
        //有待办节点的(不包含退回)
        List<ProcessTaskVo> todoList = processTaskService.getUserTodoList(req);
        //有退回待办的
        List<ProcessTaskVo> backList = processTaskService.getBackTodoList(req);
        //合并 并且排序
        List<ProcessTaskVo> list = Stream.concat(todoList.stream(),backList.stream())
                .filter(Objects::nonNull)
                .sorted(Comparator.comparing(ProcessTaskVo::getTaskCreateTime)
                        .reversed())
                .collect(Collectors.toList());

        if (CollUtil.isEmpty(list)) {
            return PageVo.empty();
        }

        List<ProcessTaskVo> userTodoList = list.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        Set<String> processSet = Sets.newHashSet();
        Map<String, ProcessTaskVo> taskMap = userTodoList.stream()
                .sorted((t1,t2) -> t2.getTaskCreateTime().compareTo(t1.getTaskCreateTime()))
                .collect(Collectors.toList()).stream()
                .filter(v -> processSet.add(v.getNodeId()))
                .collect(Collectors.toMap(ProcessTaskVo::getNodeId, v -> v));



        List<ResToBeProcessedVO> resVos = userTodoList.stream().map(d -> {
            Project projectInfo = projectInfoMap.get(d.getInstanceId());
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            BeanUtils.copyProperties(projectInfo, res);
            res.setInstCode(d.getInstanceId());
            res.setProjectId(projectInfo.getId());
            res.setBuildOrg(projectInfo.getBuildOrgName());
            res.setDeclaredAmount(projectInfo.getDeclareAmount());
            ProcessTaskVo taskVo = taskMap.get(d.getNodeId());
            res.setNodeId(d.getNodeId());
            res.setProcessStatusName(taskVo.getStatus());
            res.setProcessLaunchTime(d.getTaskCreateTime());
            res.setProcessDefName(d.getProcessDefName());
            res.setTaskId(d.getTaskId());
            res.setNodeName(findSubProcessName(d));
            res.setPretrialFileId(projectInfo.getPretrialFileId());
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos, list.size());
    }

    private String findSubProcessName(ProcessTaskVo d) {
        if(StringUtils.isBlank(d.getProcessDefId())){
            return null;
        }
        BpmnModel bpmnModel = repositoryService.getBpmnModel(d.getProcessDefId());

        if(Objects.nonNull(bpmnModel)){
            FlowElement flowElement = bpmnModel.getFlowElement(d.getTaskDefKey());
            FlowElementsContainer parentContainer = flowElement.getParentContainer();
            if(parentContainer instanceof SubProcess){
                return ((SubProcess) parentContainer).getName();
            }
        }
        return null;
    }

    /**
     * 待办中心列表导出
     *
     * @param response
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/01
     */
    public void exportTodoList(HttpServletResponse response, ToBeProcessedExportReq param) {

        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取用户全量用户信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        List<ProcessTaskVo> userTodoList = Lists.newArrayList();
        Map<String, Project> projectInfoMap = MapUtil.newHashMap();
        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        if (CollUtil.isNotEmpty(projects)) {
            Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
            // 再查出项目关联的流程实例ID
            List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
            List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIdList).orderByDesc(ProjectInst::getProjectId));
            projectInfoMap = projectInstList.stream()
                    .collect(Collectors.toMap(ProjectInst::getInstCode, p -> projectsMap.get(p.getProjectId())));
            List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());


            // 查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setInstCodes(instCodes);
            req.setProcessDefId(param.getProcessDefId());
            req.setEmployeeCode(employeeCode);

            // 有待办节点的(不包含退回)
            List<ProcessTaskVo> todoList = processTaskService.getUserTodoList(req);
            // 有退回待办的
            List<ProcessTaskVo> backList = processTaskService.getBackTodoList(req);
            // 合并 并且排序
            userTodoList = Stream.concat(todoList.stream(), backList.stream()).filter(Objects::nonNull)
                    .sorted(Comparator.comparing(ProcessTaskVo::getTaskCreateTime).reversed()).collect(Collectors.toList());
        }
        ExcelExportWriter excelExportWriter = new ExcelExportWriter();
        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_待我处理_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_待我处理_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);
        List<String> sheetsNames = Lists.newArrayList(fileName);
        List<List<String>> exportDatas = ExcelDownUtil.getTaskExportDatas(param.getExportOptionList(), userTodoList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = Lists.newArrayList();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);
    }

    /**
     * 流程处理相关操作
     *
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/01
     */
    public String handler(ReqProcessHandlerDTO param) {
        // 获取登录用户ID
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        // 获取登录用户浙政钉code
        String employeeCode = user.getEmployeeCode();
        // 获取入参
        String processInstanceId = param.getInstanceId();
        Long projectId = param.getProjectId();
        VUtils.isTrue(Objects.isNull(processInstanceId)).throwMessage("获取流程实例ID失败!");
        VUtils.isTrue(Objects.isNull(projectId)).throwMessage("获取项目ID失败!");

        // 获取当前申报项目
        Project declaredProject = projectService.getOne(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getInstCode, processInstanceId)
                .eq(Project::getId, projectId));
        VUtils.isTrue(Objects.isNull(declaredProject)).throwMessage("获取项目失败!");

        // 获取当前未处理流程详情
        ProcessProgressVo currentInstanceDetail = processInstanceService.getProgressInstanceDetail(null, processInstanceId);

        // 获取当前要处理的流程实例
        HistoricProcessInstance instance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(processInstanceId)
                .singleResult();

        //提前判断下 是不是 已经被退回了
        checkIsBack(instance);

        //进入处理逻辑
        switch (param.getAction()) {
            // 通过
            case PASS:
            // 盖章并通过
            case SEAL_PASS:
                //前置判断前端传的taskId是否存在不存在的情况 有可能是 或签被别人审批掉了
                Task task = taskService.createTaskQuery().taskId(param.getTaskId()).active().singleResult();
                if (ObjectUtil.isNull(task)) {
                    return TodoCenterContant.Handler.TASKID_ERROR;
                }
                String nodeId = task.getTaskDefinitionKey();
                // 通过该任务，流程到下一审核人处
                processTaskService.handleTask(param, employeeCode);
                //通过审核后 所处理的逻辑
                handlerManage.afterPassTodo(declaredProject,instance);

                return findNextTaskId(instance,employeeCode,nodeId,param.getTaskId());
            // 驳回
            case REJECT:
                //前置判断前端传的taskId是否存在不存在的情况 有可能是 或签被别人审批掉了
                task = taskService.createTaskQuery().taskId(param.getTaskId()).active().singleResult();
                if (ObjectUtil.isNull(task)) {
                    return TodoCenterContant.Handler.TASKID_ERROR;
                }
                nodeId = task.getTaskDefinitionKey();
                // 驳回该任务，中止流程并使项目进入对应状态,给项目创建人、流程发起人发送浙政钉工作通知：
                // 【项目名称】的【流程名称】被驳回，请及时处理。
                processTaskService.handleTask(param, employeeCode);
                //驳回审核后 所处理的逻辑
                handlerManage.afterRejectTodo(declaredProject,instance);
                break;
            // 退回
            case BACK:
                //前置判断前端传的taskId是否存在不存在的情况 有可能是 或签被别人审批掉了
                task = taskService.createTaskQuery().taskId(param.getTaskId()).active().singleResult();
                if (ObjectUtil.isNull(task)) {
                    return TodoCenterContant.Handler.TASKID_ERROR;
                }
                nodeId = task.getTaskDefinitionKey();
                // 退回该任务
                processTaskService.handleTaskLs(param, employeeCode);
                //退回审核后 所处理的逻辑
                handlerManage.afterBackTodo(declaredProject,instance);
                break;
            // 撤回(流程发起人和当前流程审核人的前一个审核人操作)
            case WITHDRAW:
                /**
                 * 新逻辑
                 * 不判断前端传的taskId
                 * 1.先判断 用户是否是ROOT 发起人 并且流程没有开始审批
                 */
                if(withDrawHandle.canRootWithDraw(instance,employeeCode)){
                    // 登录用户是流程发起人，且是流程发起人撤回
                    processTaskService.rootWithdrawTask(param, employeeCode);
                    //发起人撤回
                    handlerManage.rootWithDraw(declaredProject);
                }else{
                    // 2.如果用户不是ROOT发起人 或者 不满足root撤回 那么必定要判断 他是不是上个节点审批人 或者 当前会签已审批的审批人
                    // 这里有个小操作 check的同时 把对应的操作人的taskId 也塞入
                    VUtils.isTrue(!withDrawHandle.checkUserIsBefore(currentInstanceDetail.getProgressInfo(),param))
                            .throwMessage("当前登录用户不是发起人,也不是上一个节点审批人或者当前会签已批审批人 无法进行撤回操作!");

                    //前置判断前端传的taskId是否存在不存在的情况 有可能是 或签被别人审批掉了
                    HistoricTaskInstance taskInst = historyService.createHistoricTaskInstanceQuery().taskId(param.getTaskId()).finished().singleResult();
                    if (ObjectUtil.isNull(taskInst)) {
                        return TodoCenterContant.Handler.TASKID_ERROR;
                    }
                    nodeId = taskInst.getTaskDefinitionKey();

                    //上个审批人处理逻辑
                    processTaskService.lastWithdrawTask(param);

                    return findNextTaskId(instance,employeeCode,nodeId,param.getTaskId());
                }
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + param.getAction());
        }
        return "操作成功";
    }

    private void checkIsBack(HistoricProcessInstance instance) {
        VUtils.isTrue(Objects.nonNull(instance.getBusinessStatus()) &&
                        HisProInsEndActId.BACK.equals(instance.getBusinessStatus()))
                .throwMessage("该审核流程已被退回 暂时无法操作!");
    }

    /**
     * 调用IRS接口,获取盖章后的pdf文件,上传到OSS，并保存文件ID到项目库中
     * @param req
     * @param declaredProject
     * @return void
     * @author CMM
     * @since 2023/03/16 21:34
     */
    private Long getSealedPdf(SealInfoDTO req, Project declaredProject) {
        Long projectId = req.getProjectId();
        // 设置pdf模板参数
        JSONObject paramsMap = assemblyPdfParams(declaredProject);
        // TODO 根据当前盖章单位设置对应的盖章意见与盖章日期
        paramsMap.put("superOrgOpinion",null);
        paramsMap.put("superOrgAuditDate",null);
        paramsMap.put("higherOrgOpinion",null);
        paramsMap.put("higherOrgAuditDate",null);
        paramsMap.put("bigDataBureauOpinion",null);
        paramsMap.put("bigDataBureauAuditDate",null);
        // 再次获取装配了盖章意见和盖章日期参数的未盖章pdf文件
        Long pretrialFileId = generateNotSealedPdf(projectId,paramsMap);
        // 获取预审申请单文件,并转换为Base64
        File pdfFile = fileService.getById(pretrialFileId);
        InputStream fileInputStream = fileService.getFileInputStream(pdfFile);
        String pdfEncode64 = Base64.encode(fileInputStream);
        // 装配盖章接口请求参数信息
        SignReqDTO signReq = new SignReqDTO();
        signReq.setPdfEncode64(pdfEncode64);
        String originalFileName = pdfFile.getOriginalFileName();
        signReq.setFileName(originalFileName);
        Float posX = req.getPosX();
        Float posY = req.getPosY();
        signReq.setPosX(posX);
        signReq.setPosY(posY);
        // 获取当前登录签章用户信息
        Long userId = LoginUserUtil.getUserId();
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取印章编号
        CompanySignature companySignature = companySignatureService.getOne(Wrappers.lambdaQuery(CompanySignature.class)
                .eq(CompanySignature::getOrganizationCode, userFullInfo.getEmpPosUnitCode()));
        String sealSn = companySignature.getSealSn();
        signReq.setSealSn(sealSn);
        String posPage = req.getPosPage();
        signReq.setPosPage(posPage);
        Integer signType = req.getSignType();
        signReq.setSignType(signType);
        // 调用盖章接口,获取盖章后返回的pdf文件字符数组
        String signPdf = IRSAPIRequest.createSignPdf(signReq);
        ByteArrayInputStream inputStream = null;
        if (Objects.nonNull(signPdf)) {
            BASE64Decoder decoder = new BASE64Decoder();
            try {
                //转码
                byte[] b = decoder.decodeBuffer(signPdf);
                for (int i = 0; i < b.length; ++i) {
                    if (b[i] < 0) {//调整异常数据
                        b[i] += 256;
                    }
                }
                inputStream = new ByteArrayInputStream(b);
            }catch (Exception e){

            }
        }
        // 转换成MultipartFile
        MultipartFile multipartFile;
        try {
            multipartFile = new MockMultipartFile("file", originalFileName, "application/pdf", inputStream);
        } catch (IOException e) {
            throw new BizException("生成盖章pdf文件失败!");
        }
        // 上传OSS
        FileResultVO resultVo = fileService.upload(multipartFile, "default");
        // 将返回的文件ID更新到项目库中对应的项目下
        declaredProject.setPretrialFileId(resultVo.getId());
        declaredProject.setPretrialFileName(resultVo.getOriginalFileName());
        declaredProject.setUpdateOn(LocalDateTime.now());
        projectService.updateById(declaredProject);
        return resultVo.getId();
    }

    /**
     * 查询流程表单数据及审批的进度步骤
     *
     * @param request 请求参数
     * @return 流程进度及表单详情
     */
    public ProcessProgressDetailVo getProcessDetail(ProcessDetailReq request) {
        String employeeCode = LoginUserUtil.loginUserDetail().getEmployeeCode();
        String instanceId = request.getInstanceId();
        String nodeId = request.getNodeId();
        Long projectId = request.getProjectId();
        ProcessProgressVo progressInstanceDetail = processInstanceService.getProgressInstanceDetail(nodeId, instanceId);

        if(Objects.isNull(progressInstanceDetail)){
            return null;
        }

        List<ProgressNode> progressInfo = progressInstanceDetail.getProgressInfo();
        if (CollUtil.isNotEmpty(progressInfo)){
            buildUserUtils.buildUserByProcessInfo(progressInfo);
        }
        ProcessProgressDetailVo res = new ProcessProgressDetailVo();
        res.setProcessProgressVo(progressInstanceDetail);
        res.setStatus(progressInstanceDetail.getStatus());
        res.setProjectId(projectId);
        res.setCanWithdraw(withDrawHandle.checkCanWithdraw(instanceId,progressInstanceDetail,request.getTaskId()));
        passHandle.checkCanPassOrSeal(request.getInstanceId(),request.getTaskId(),employeeCode,res);
        return res;
    }


    /**
     * 待办中心我已处理项目列表查询
     *
     * @param param
     * @author CMM
     * @since 2023/02/03
     */
    public PageVo<ResToBeProcessedVO> idoList(ToBeProcessedReq param) {

        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
        if (CollUtil.isEmpty(projectIdList)) {
            return PageVo.empty();
        }
        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        req.setInstCodes(instCodes);
        List<ProcessTaskVo> iDoList = processTaskService.getUserIdoList(req);
        if (CollUtil.isEmpty(iDoList)) {
            return PageVo.empty();
        }

        List<ProcessTaskVo> userIdoList = iDoList.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        List<ResToBeProcessedVO> resVos = userIdoList.stream().map(d -> {
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            Project projectInfo = projectInfoMap.get(d.getInstanceId());
            BeanUtils.copyProperties(projectInfo, res);
            res.setInstCode(d.getInstanceId());
            res.setBuildOrg(projectInfo.getBuildOrgName());
            res.setDeclaredAmount(projectInfo.getDeclareAmount());
            res.setProjectId(projectInfo.getId());
            res.setProcessStatusName(d.getStatus());
            res.setProcessLaunchTime(d.getCreateTime());
            res.setProcessHandleTime(d.getTaskEndTime());
            res.setTaskId(d.getTaskId());
            res.setNodeName(findSubProcessName(d));
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos, iDoList.size());
    }

    /**
     * 导出我已处理项目列表
     *
     * @param response 返回数据流
     * @param param    请求参数
     * @return void
     * @author CMM
     * @since 2023/02/06
     */
    public void idoExport(HttpServletResponse response, ToBeProcessedExportReq param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        //因为要解耦 不能把项目和工作流一起查
        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        List<ProcessTaskVo> userIdoList = Lists.newArrayList();
        Map<String, Project> projectInfoMap = Maps.newHashMap();
        if (CollUtil.isNotEmpty(projects)) {
            Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
            // 再查出项目关联的流程实例ID
            List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());

            List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIdList)
                    .orderByDesc(ProjectInst::getProjectId));
            projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
            List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

            // 查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setProcessDefId(param.getProcessDefId());
            req.setUserId(String.valueOf(userId));
            req.setEmployeeCode(employeeCode);
            req.setInstCodes(instCodes);
            userIdoList = processTaskService.getUserIdoList(req);
        }

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我已处理_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我已处理_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);

        List<List<String>> exportDatas = ExcelDownUtil.getTaskExportDatas(param.getExportOptionList(), userIdoList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);

    }

    /**
     * 查询我提交的项目列表
     *
     * @param param
     * @return com.ningdatech.basic.model.PageVo<com.ningdatech.pmapi.todocenter.model.dto.res.ResToBeProcessedDTO>
     * @author CMM
     * @since 2023/02/06
     */
    public PageVo<ResToBeProcessedVO> mySubmittedList(ToBeProcessedReq param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());

        if (CollUtil.isEmpty(projectIdList)) {
            return PageVo.empty();
        }

        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        req.setInstCodes(instCodes);
        List<ProcessInstanceVo> submittedList = processInstanceService.getUserSubmittedList(req);

        if (CollUtil.isEmpty(submittedList)) {
            return PageVo.empty();
        }

        List<ProcessInstanceVo> userSubmittedList = submittedList.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        List<ResToBeProcessedVO> resVos = userSubmittedList.stream().map(d -> {
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            Project project = projectInfoMap.get(d.getInstanceId());
            BeanUtils.copyProperties(project, res);
            res.setInstCode(d.getInstanceId());
            res.setBuildOrg(project.getBuildOrgName());
            res.setDeclaredAmount(project.getDeclareAmount());
            res.setProjectId(project.getId());
            res.setProcessStatusName(d.getStatus());
            res.setProcessLaunchTime(d.getStartTime());
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos,submittedList.size());
    }

    /**
     * 待办中心-我发起的项目列表导出
     *
     * @param response
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/11
     */
    public void exportMySubmittedList(HttpServletResponse response, ToBeProcessedExportReq param) {

        // 获取登录用户ID
        long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);

        List<ProcessInstanceVo> userSubmittedList = Lists.newArrayList();
        Map<String, Project> projectInfoMap = Maps.newHashMap();
        if (CollUtil.isNotEmpty(projects)) {
            Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
            // 再查出项目关联的流程实例ID
            List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
            List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIdList)
                    .orderByDesc(ProjectInst::getProjectId));
            projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
            List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

            // 查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setProcessDefId(param.getProcessDefId());
            req.setUserId(String.valueOf(userId));
            req.setEmployeeCode(employeeCode);
            req.setInstCodes(instCodes);
            userSubmittedList = processInstanceService.getUserSubmittedList(req);
        }

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我发起的_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我发起的_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getInstExportDatas(param.getExportOptionList(), userSubmittedList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);

    }

    /**
     * 待办中心-抄送我的项目列表查询
     *
     * @param param
     * @return com.ningdatech.basic.model.PageVo<com.ningdatech.pmapi.todocenter.model.dto.res.ResToBeProcessedDTO>
     * @author CMM
     * @since 2023/02/11
     */
    public PageVo<ResToBeProcessedVO> ccmeList(ToBeProcessedReq param) {

        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);

        if (CollUtil.isEmpty(projects)) {
            return PageVo.empty();
        }

        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        req.setInstCodes(instCodes);
        List<ProcessInstanceVo> ccMeList = processInstanceService.getCcMeList(req);

        if (CollUtil.isEmpty(ccMeList)) {
            return PageVo.empty();
        }

        List<ProcessInstanceVo> ccMeInstList = ccMeList.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        List<ResToBeProcessedVO> resVos = ccMeInstList.stream().map(d -> {
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            Project project = projectInfoMap.get(d.getInstanceId());
            BeanUtils.copyProperties(project, res);
            res.setInstCode(d.getInstanceId());
            res.setNodeId(d.getNodeId());
            res.setProjectId(project.getId());
            res.setBuildOrg(project.getBuildOrgName());
            res.setDeclaredAmount(project.getDeclareAmount());
            res.setProcessStatusName(d.getStatus());
            res.setProcessLaunchTime(d.getStartTime());
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos, ccMeList.size());
    }

    /**
     * 待办中心-抄送我的项目列表导出
     *
     * @param response
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/11
     */
    public void exportCcMeProjectList(HttpServletResponse response, ToBeProcessedExportReq param) {

        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);

        List<ProcessInstanceVo> ccMeList = Lists.newArrayList();
        Map<String, Project> projectInfoMap = Maps.newHashMap();
        if (CollUtil.isNotEmpty(projects)) {
            Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
            // 再查出项目关联的流程实例ID
            List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
            List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIdList)
                    .orderByDesc(ProjectInst::getProjectId));
            projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
            List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

            // 分页查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setProcessDefId(param.getProcessDefId());
            req.setUserId(String.valueOf(userId));
            req.setEmployeeCode(employeeCode);
            req.setInstCodes(instCodes);
            ccMeList = processInstanceService.getCcMeList(req);
        }

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_抄送我的_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_抄送我的_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getInstExportDatas(param.getExportOptionList(), ccMeList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);

    }

    /**
     * 被退回项目内容调整并重新通过
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String adjustmentAndHandle(AdjustHandleDTO dto) {
        ProjectDTO projectDto = dto.getProjectInfo();
        Project projectInfo = projectService.getById(projectDto.getId());

        VUtils.isTrue(Objects.isNull(projectInfo)).throwMessage("调整失败 此项目不存在!");

        //项目名称去重
        //项目名称去重
        if(StringUtils.isNotBlank(projectDto.getProjectName()) &&
                !projectDto.getProjectName().equals(projectInfo.getProjectName())){
            projectDto.setProjectCode(projectInfo.getProjectCode());
            defaultDeclaredProjectManage.checkDuplication(projectDto);
        }
        defaultDeclaredProjectManage.checkAmount(projectDto);

        //修改项目内容
        if(!modifyProject(projectDto)){
            throw new BusinessException("调整项目失败!");
        }

        //设置 节点的状态 为null 代表继续审核
        List<HistoricVariableInstance> approves = historyService.createHistoricVariableInstanceQuery()
                .processInstanceId(dto.getInstanceId())
                .variableNameLike("approve_%")
                .list();
        //删除 退回的评论和意见
        handlerManage.deleteBackComments(approves);

        // 更新当前流程状态为审核中
        processTaskService.updateProInstStatus(dto.getInstanceId());

        return dto.getInstanceId();
    }

    private Boolean modifyProject(ProjectDTO projectDto) {
        //先修改项目信息
        Project project = new Project();
        BeanUtils.copyProperties(projectDto,project);
        project.setUpdateOn(LocalDateTime.now());
        if(!projectService.updateById(project)){
            throw new BusinessException("项目调整失败");
        }
        //再修改应用信息
        Boolean isApp = Objects.nonNull(projectDto.getIncludeApplication()) && 1 == projectDto.getIncludeApplication()
                ? Boolean.TRUE : Boolean.FALSE;
        //批量删除
        projectApplicationService.remove(Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(ProjectApplication::getProjectId,project.getId()));
        if(isApp && CollUtil.isNotEmpty(projectDto.getApplicationList())){
            //批量添加
            List<ProjectApplication> applications = projectDto.getApplicationList().stream().map(application -> {
                ProjectApplication projectApplication = new ProjectApplication();
                BeanUtils.copyProperties(application, projectApplication);
                projectApplication.setProjectId(project.getId());
                return projectApplication;
            }).collect(Collectors.toList());
            projectApplicationService.saveBatch(applications);
        }
        return Boolean.TRUE;
    }

    /**
     * 获取未盖章的预审申请单pdf文件
     * @param projectId
     * @return java.lang.Long
     * @author CMM
     * @since 2023/03/16 22:46
     */
    public Long getNotSealedPdf(Long projectId) {
        Project project = projectService.getById(projectId);
        Long pretrialFileId = project.getPretrialFileId();
        // 如果项目关联的预审文件ID不为空，说明已经进行过盖章审核，直接返回盖章后的预审申请单文件ID
        if (Objects.nonNull(pretrialFileId)){
            return pretrialFileId;
        }
        // 设置pdf模板参数
        JSONObject paramsMap = assemblyPdfParams(project);
        paramsMap.put("superOrgOpinion",null);
        paramsMap.put("superOrgAuditDate",null);
        paramsMap.put("higherOrgOpinion",null);
        paramsMap.put("higherOrgAuditDate",null);
        paramsMap.put("bigDataBureauOpinion",null);
        paramsMap.put("bigDataBureauAuditDate",null);
        return generateNotSealedPdf(projectId, paramsMap);
    }

    private Long generateNotSealedPdf(Long projectId, JSONObject paramsMap) {
        // 获取本地目录的pdf模板
        String fileName = "预审申请单";
        InputStream htmlInputStream =
                this.getClass().getClassLoader().getResourceAsStream("template" + java.io.File.separator + fileName + ".html");
        // 生成pdf字节数组
        byte[] pdf = pdfUtils.generatePdf(htmlInputStream, paramsMap);

        // 转换成MultipartFile
        MultipartFile multipartFile = new MockMultipartFile("file", fileName + ".pdf", "application/pdf", pdf);

        // 上传OSS
        FileResultVO resultVO = fileService.upload(multipartFile, "default");

        // 将返回的文件ID保存到项目库中
        Project project = projectService.getById(projectId);
        project.setPretrialFileId(resultVO.getId());
        project.setPretrialFileName(resultVO.getOriginalFileName());
        project.setUpdateOn(LocalDateTime.now());
        projectService.updateById(project);
        try {
            if (null != htmlInputStream) {
                htmlInputStream.close();
            }

            // 删除输出文件夹下的临时文件
            java.io.File directory = new java.io.File("");
            //pdf输出路径
            String filePath = directory.getAbsolutePath() + java.io.File.separator + "fileout";
            java.io.File directoryPath = new java.io.File(filePath);
            FileUtils.cleanDirectory(directoryPath);
        } catch (IOException e) {
            log.error(e.getMessage());
        }

        // 返回文件id供前端下载
        return resultVO.getId();
    }

    /**
     * 装配pdf模板参数
     * @param project 申报项目ID
     * @return com.alibaba.fastjson.JSONObject
     * @author CMM
     * @since 2023/03/16 22:28
     */
    private JSONObject assemblyPdfParams(Project project) {
        // 获取本地目录的pdf模板
        String fileName = "预审申请单";
        Long projectId = project.getId();
        // 获取表单数据
        PdfGenerateDTO pdfGenerateDTO = new PdfGenerateDTO();
        BeanUtils.copyProperties(project, pdfGenerateDTO);

        JSONObject paramsMap = JSONObject.parseObject(JSONObject.toJSONString(pdfGenerateDTO));
        // 设置title
        String title = "【" + project.getProjectName() + "】" + fileName;
        paramsMap.put("title",title);
        // 获取预审申报的开始时间
        ProjectInst projectInst = projectInstService.getOne(Wrappers.lambdaQuery(ProjectInst.class)
                .eq(ProjectInst::getProjectId, projectId)
                .eq(ProjectInst::getInstType, InstTypeEnum.PRELIMINARY_PREVIEW.getCode())
                .orderByDesc(ProjectInst::getCreatOn)
                .last("limit 1"));
        LocalDateTime time = null;
        if (Objects.nonNull(projectInst)) {
            time = projectInst.getCreatOn();
            paramsMap.put("time", NdDateUtils.format(time, "yyyy-MM-dd HH:mm"));
        }else {
            paramsMap.put("time", null);
        }
        // 获取是否临时增补项目
        String isTemporaryAugment = IsOrNotEnum.getDescByCode(project.getIsTemporaryAugment());
        paramsMap.put("isTemporaryAugment", isTemporaryAugment);
        // 获取项目类型
        Integer projectType = project.getProjectType();
        paramsMap.put("projectType", ProjectTypeEnum.getDesc(projectType));
        // 获取是否首次新建
        String isFirst = IsOrNotEnum.getDescByCode(project.getIsFirst());
        paramsMap.put("isFirst", isFirst);
        // 获取是否一地创新全省共享项目
        Boolean isInnovateWholeProvinceShare = project.getIsInnovateWholeProvinceShare();
        paramsMap.put("isInnovateWholeProvinceShare", Boolean.TRUE.equals(isInnovateWholeProvinceShare) ?
                IsOrNotEnum.IS.getDesc() : IsOrNotEnum.NOT.getDesc());
        // 获取项目建设起始时间
        String beginTime = project.getBeginTime();
        String endTime = project.getEndTime();
        String beginAndEndTime = beginTime + StrUtil.DASHED + endTime;
        paramsMap.put("beginAndEndTime", beginAndEndTime);
        // 获取四大体系
        String fourSystem = FourSystemEnum.getDescByCode(project.getFourSystems());
        paramsMap.put("fourSystems", fourSystem);
        // 获取是否数字化改革项目
        String isDigitalReform = IsOrNotEnum.getDescByCode(project.getIsDigitalReform());
        paramsMap.put("isDigitalReform",isDigitalReform);
        if (IsOrNotEnum.NOT.getDesc().equals(isDigitalReform)){
            paramsMap.put("bizDomain", null);
        }else {
            paramsMap.put("bizDomain", BizDomainEnum.getDescByCode(project.getBizDomain()));
        }
        // 获取是否上云
        String isCloud = IsOrNotEnum.getDescByCode(project.getIsCloud());
        paramsMap.put("isCloud", isCloud);
        if (IsOrNotEnum.NOT.getDesc().equals(isCloud)){
            paramsMap.put("cloudType", null);
        }else {
            String cloudType = project.getCloudType();
            String cloudTypeName;
            // 云类型为多选
            if (cloudType.contains(StrPool.COMMA)){
                cloudTypeName = Arrays.stream(cloudType.split(StrPool.COMMA))
                        .map(s -> CloudTypeEnum.getDescByCode(Integer.valueOf(s)))
                        .collect(Collectors.joining(StrPool.COMMA));

            }else {
                cloudTypeName = CloudTypeEnum.getDescByCode(Integer.valueOf(cloudType));
            }
            paramsMap.put("cloudType",cloudTypeName);
        }
        // 获取本年计划投资金额
        BigDecimal yearPlanInvest = project.getAnnualPlanAmount();
        paramsMap.put("yearPlanInvest",yearPlanInvest);
        // TODO 根据地区编码获取当地大数据局(中心)的名称
        paramsMap.put("bigDataBureauName",null);
        // 备注模块如果为空
        if (Objects.isNull(pdfGenerateDTO.getProjectRemarks())){
            paramsMap.put("projectRemarks",null);
        }

        // 获取立项依据
        String buildBasis = project.getBuildBasis();
        StringBuilder builder = new StringBuilder();
        List<JSONObject> fileArray = JSON.parseArray(buildBasis, JSONObject.class);
        for (JSONObject jsonObject : fileArray) {
            String titleStr = jsonObject.getString(CommonConst.TITLE);
            builder.append(CommonConst.ITEM_BASED);
            builder.append(titleStr);
            builder.append(StrPool.SPACE);
            String fileNameStr = jsonObject.getString(CommonConst.FILE_NAME_STR);
            builder.append(CommonConst.ITEM_BASED_FILE_NAME);
            builder.append(fileNameStr);
            builder.append(StrPool.SPACE);
            String fileInfo = jsonObject.getString(CommonConst.FILE_LIST);
            List<JSONObject> fileInfoArray = JSON.parseArray(fileInfo, JSONObject.class);
            List<String> nameList = CollUtils.fieldList(fileInfoArray, w -> w.getString(CommonConst.FILE_NAME));
            String appendixStr = nameList.stream().collect(Collectors.joining(StrPool.COMMA));
            builder.append(CommonConst.APPENDIX);
            builder.append(appendixStr);
            builder.append(StrPool.SEMICOLON);
            builder.append(StrPool.NEWLINE);
        }
        String buildBasisList = builder.toString();
        paramsMap.put("buildBasisList",buildBasisList);

        return paramsMap;
    }

    public Long getSealedPdf(SealInfoDTO req) {
        Long projectId = req.getProjectId();
        Project declaredProject = projectService.getById(projectId);
        return getSealedPdf(req, declaredProject);
    }

    //获取当前登录人的 待办中心统计数据
    public TodoCenterStatisticsVO todoCenterStatistics(String employeeCode){
        //为了查询效率 还是自己去写统计接口
        TodoCenterStatisticsPO statistics = statisticsService.getStatistics(employeeCode);
        TodoCenterStatisticsVO statisticsVO = BeanUtil.copyProperties(statistics,TodoCenterStatisticsVO.class);

        return statisticsVO;
    }

    public TodoNumVO getTodoNums(ToBeProcessedReq param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取员工浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();
        TodoNumVO todoNumVo = new TodoNumVO();
        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        if(CollUtil.isEmpty(projects)){
            return todoNumVo;
        }
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());

        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());
        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        req.setInstCodes(instCodes);
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        //有待办节点的(不包含退回)
        List<ProcessTaskVo> todoList = processTaskService.getUserTodoList(req);
        //有退回待办的
        List<ProcessTaskVo> backList = processTaskService.getBackTodoList(req);
        //合并 并且排序
        List<ProcessTaskVo> list = Stream.concat(todoList.stream(),backList.stream())
                .filter(Objects::nonNull)
                .sorted(Comparator.comparing(ProcessTaskVo::getTaskCreateTime)
                        .reversed())
                .collect(Collectors.toList());
        if (CollUtil.isEmpty(list)){
            return todoNumVo;
        }

        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        Map<String, Project> projectInfoMap = projectInstList.stream()
                .collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        // 关联项目信息
        List<Project> projectList = list.stream()
                .map(d -> projectInfoMap.get(d.getInstanceId()))
                .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(projectList)) {
            todoNumVo.setTotalNum(projectList.size());
        }
        // 筛选出增补项目和非增补项目
        List<Project> appendProjects = projectList.stream()
                .filter(p -> IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(p.getIsTemporaryAugment()))
                .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(appendProjects)){
            todoNumVo.setAppendNum(appendProjects.size());
        }
        List<Project> notAppendProjects = projectList.stream()
                .filter(p -> IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(p.getIsTemporaryAugment()))
                .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(notAppendProjects)){
            todoNumVo.setNotAppendNum(notAppendProjects.size());
        }
        return todoNumVo;
    }

    /**
     * 寻找下一个审核task
     * @param instance
     * @return
     */
    private String findNextTaskId(HistoricProcessInstance instance,String employeeCode,String nodeId,String taskId) {
        BpmnModel bpmnModel = repositoryService.getBpmnModel(instance.getProcessDefinitionId());
        Map<String, String> subNodeMap = ProcessTaskUtils.getSubNodeMap(bpmnModel);
        String subProcessId = subNodeMap.get(nodeId);
        Map<String, List<String>> allSubNodeMap = ProcessTaskUtils.loadSubNodeMap(bpmnModel);
        List<String> nodeIds = allSubNodeMap.get(subProcessId);

        List<Task> tasks = taskService.createTaskQuery()
                .processInstanceId(instance.getId())
                .taskAssignee(employeeCode)
                .taskDefinitionKeys(nodeIds)
                .orderByTaskCreateTime()
                .asc()
                .list();
        if(CollUtil.isEmpty(tasks)){
            //如果此子单位为空 那就直接再查下 他的其它任务
            List<Task> currentTasks = taskService.createTaskQuery()
                    .processInstanceId(instance.getId())
                    .taskAssignee(employeeCode)
                    .orderByTaskCreateTime()
                    .asc()
                    .list();
            if(CollUtil.isNotEmpty(currentTasks)){
                return currentTasks.get(0).getId();
            }
            return taskId;
        }
        return tasks.get(0).getId();
    }

    /**
     * 取当前用户的下一个taskId
     * @param instanceId
     * @return
     */
    public String todoTaskId(String instanceId) {
        String employeeCode = LoginUserUtil.loginUserDetail().getEmployeeCode();
        List<Task> tasks = taskService.createTaskQuery()
                .processInstanceId(instanceId)
                .taskAssignee(employeeCode)
                .orderByTaskCreateTime()
                .asc()
                .list();
        if(CollUtil.isEmpty(tasks)){
            return null;
        }
        return tasks.get(0).getId();
    }
}
